@@ -15,16 +15,16 @@
  */
 /*
  * File: audiocd.h
- * Created by: luke_biddell@yahoo.com
+ * Copyright: luke_biddell@yahoo.com
  * Created on: Fri May  7 22:37:25 2004
  */
  
 #include "audiocd.h"
 #include "gnomebaker.h"
 #include "gbcommon.h"
-#include "audioinfo.h"
 #include "burn.h"
 #include <stdio.h>
+#include "media.h"
 
 
 gdouble audiocdsize = 0.0;
@@ -33,14 +33,15 @@ DiskSize audiodisksizes[] =
 {
 	{22, "22 min. CD"},
 	{74, "74 min. CD"},
-	{80, "80 min. CD"}
+	{80, "80 min. CD"},
+	{90, "90 min. CD"}
 };
 
 
 enum
 {
     TARGET_STRING,
-    TARGET_URL
+    TARGET_URL    
 };
 
 
@@ -52,6 +53,37 @@ static GtkTargetEntry targetentries[] =
 };
 
 
+void 
+audiocd_move_selected(const gboolean up)
+{
+    GB_LOG_FUNC
+    
+    GtkWidget* tree = glade_xml_get_widget(gnomebaker_getxml(), widget_audiocd_tree);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));        
+    GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
+    GList* list = gtk_tree_selection_get_selected_rows(selection, &model);    
+    GList* current = NULL; 
+    for((up ? (current = list) : (current = g_list_last(list))); 
+        current != NULL; 
+        (up ? (current = current->next) : (current = current->prev)))
+    {
+        GtkTreeIter from, to;
+        GtkTreePath* path = (GtkTreePath*)current->data;
+        gtk_tree_model_get_iter(model, &from, path);
+        
+        if(up) gtk_tree_path_prev(path);
+        else gtk_tree_path_next(path);
+            
+        if(gtk_tree_model_get_iter(model, &to, path))
+            gtk_list_store_swap(GTK_LIST_STORE(model), &from, &to);
+            
+        gtk_tree_path_free(path);     
+    }
+    
+    g_list_free (list);        
+}
+
+
 gint 
 audiocd_get_audiocd_size()
 {
@@ -108,7 +140,7 @@ audiocd_update_progress_bar(gboolean add, gdouble seconds)
 	}
 	else
 	{
-		gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
+		gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
 			_("Track is too large to fit in the remaining space on the CD"));
 		ok = FALSE;
 	}
@@ -149,7 +181,7 @@ audiocd_remove()
 	gtk_tree_selection_selected_foreach(selection, audiocd_foreach_fileselection, &rr_list);		
 	
 	GList *node = rr_list;
-	while(node != NULL)
+	for (; node != NULL ; node = node->next)
 	{
 		if(node->data)
 		{
@@ -188,7 +220,6 @@ audiocd_remove()
 				/* FIXME/CHECK: Do we need to free the path here? */
 			}
 		}
-		node = node->next;
 	}
 	
 	g_list_foreach(rr_list, (GFunc)gtk_tree_row_reference_free, NULL);
@@ -235,26 +266,71 @@ audiocd_on_clear_clicked(GtkWidget *menuitem, gpointer userdata)
 }
 
 
+void 
+audiocd_on_open(gpointer widget, gpointer user_data)
+{
+	GB_LOG_FUNC
+	g_return_if_fail(user_data != NULL);
+
+	GtkTreeView* view = (GtkTreeView*)user_data;	
+	GtkTreeModel* model = gtk_tree_view_get_model(view);
+	GtkTreeSelection* selection = gtk_tree_view_get_selection(view);
+	GB_DECLARE_STRUCT(GtkTreeIter, iter);
+	gtk_tree_selection_selected_foreach(selection, 
+		(GtkTreeSelectionForeachFunc)gbcommon_get_first_selected_row, &iter);	
+	gchar *file = NULL;
+	gtk_tree_model_get(model, &iter, AUDIOCD_COL_FILE, &file, -1);
+	gbcommon_launch_app_for_file(file);
+	g_free(file);
+}
+
+
+void 
+audiocd_on_move_up(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC
+    audiocd_move_selected(TRUE);
+}
+
+
+void 
+audiocd_on_move_down(gpointer widget, gpointer user_data)
+{
+    GB_LOG_FUNC
+    audiocd_move_selected(FALSE);
+}
+
+
 gboolean
 audiocd_on_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
 	GB_LOG_FUNC
+	g_return_val_if_fail(widget != NULL, FALSE);
 
 	/* look for a right click */	
 	if(event->button == 3)
 	{
-		GtkWidget* menu = gtk_menu_new();	
-		
-		GtkWidget* menuitem = gtk_menu_item_new_with_label(_("Remove selected"));	
-		g_signal_connect(menuitem, "activate",
+		GtkWidget* menu = gtk_menu_new();
+		GtkTreeView* view = (GtkTreeView*)widget;
+		GtkTreeSelection* selection = gtk_tree_view_get_selection(view);
+		const gint count = gtk_tree_selection_count_selected_rows(selection);
+		if(count == 1)
+		{
+			gbcommon_append_menu_item_stock(menu, _("_Open"), GTK_STOCK_OPEN, 
+				(GCallback)audiocd_on_open, view);
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+		}
+        
+        gbcommon_append_menu_item_stock(menu, _("Move selected _up"), GTK_STOCK_GO_UP, 
+            (GCallback)audiocd_on_move_up, widget);
+        gbcommon_append_menu_item_stock(menu, _("Move selected _down"), GTK_STOCK_GO_DOWN,
+            (GCallback)audiocd_on_move_down, widget);   
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());        
+		gbcommon_append_menu_item_stock(menu, _("_Remove selected"), GTK_STOCK_REMOVE, 
 			(GCallback)audiocd_on_remove_clicked, widget);	
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);	
-		
-		menuitem = gtk_menu_item_new_with_label(_("Clear"));	
-		g_signal_connect(menuitem, "activate",
+		gbcommon_append_menu_item_stock(menu, _("_Clear"), GTK_STOCK_CLEAR,
 			(GCallback)audiocd_on_clear_clicked, widget);	
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);	
-		
+            
 		gtk_widget_show_all(menu);
 	
 		/* Note: event can be NULL here when called. However,
@@ -264,10 +340,68 @@ audiocd_on_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer use
 					   gdk_event_get_time((GdkEvent*)event));
 		return TRUE;
 	}
-	else
-	{
-		return FALSE;
+	return FALSE;	
+}
+
+
+gboolean
+audiocd_add_file(const gchar* filename, GtkTreeModel* model)
+{
+	GB_LOG_FUNC
+	g_return_val_if_fail(filename != NULL, FALSE);
+	g_return_val_if_fail(model != NULL, FALSE);
+	
+	/* ret is set to true as we want to ignore non audio files. */
+	gboolean ret = TRUE;
+	MediaInfo* info = media_get_info(filename);
+	if(info != NULL)
+	{     
+		switch(info->status)
+        {
+            case INSTALLED:
+            {
+                if(audiocd_update_progress_bar(TRUE, (gdouble)info->duration))
+                {
+                    GdkPixbuf* icon = gbcommon_get_icon_for_mime(info->mimetype, 16);
+            
+                    GB_DECLARE_STRUCT(GtkTreeIter, iter);		
+                    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+                    gtk_list_store_set(
+                        GTK_LIST_STORE(model), &iter, 
+                        AUDIOCD_COL_ICON, icon, 
+                        AUDIOCD_COL_FILE, (gchar*)filename, 
+                        AUDIOCD_COL_DURATION, info->formattedduration->str,
+                        /*AUDIOCD_COL_SIZE, info->filesize,*/
+                        AUDIOCD_COL_ARTIST, info->artist->str, 
+                        AUDIOCD_COL_ALBUM, info->album->str,
+                        AUDIOCD_COL_TITLE, info->title->str, -1);
+                
+                    g_object_unref(icon);
+                    
+                }
+                else
+                {
+                    ret = FALSE;
+                }
+                break;
+            }
+            case NOT_INSTALLED:
+            {
+                gchar* buf = g_strdup_printf(_("The plugin to handle a file of type %s is not installed."), gbcommon_get_mime_description(info->mimetype));
+                gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, GTK_BUTTONS_NONE, buf);
+                g_free(buf);
+                break;
+            }
+            default:
+            {
+                gchar* buf = g_strdup_printf(_("There is no known plugin to handle a file of type %s."),info->mimetype);
+                gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK, GTK_BUTTONS_NONE, buf);
+                g_free(buf);
+            }
+        }
+		media_info_delete(info);
 	}
+	return ret;
 }
 
 
@@ -285,49 +419,41 @@ audiocd_add_selection(GtkSelectionData* selection)
 	
 	gnomebaker_show_busy_cursor(TRUE);
 
+	gboolean cont = TRUE;
 	const gchar* file = strtok((gchar*)selection->data, "\n");
-	while(file != NULL)
+	while((file != NULL) && cont)
 	{
 		/* Get the file name that's been dropped and if there's a 
 	   	   file url at the start then strip it off */	
-		gchar* filename = gbcommon_tidy_nautilus_dnd_file(file);
+		gchar* filename = gbcommon_get_local_path(file);		
 		
-		AudioInfo* info = audioinfo_new(filename);
-		if(info != NULL)
+		/* if the file is really a directory then we open it
+		and attempt to add any audio files in the directory */
+		if(g_file_test(filename, G_FILE_TEST_IS_DIR))
 		{
-			if(audiocd_update_progress_bar(TRUE, (gdouble)info->duration))
-			{
-				GdkPixbuf* icon = gbcommon_get_icon_for_mime(info->mimetype, 16);
-				
-				GB_DECLARE_STRUCT(GtkTreeIter, iter);		
-				gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-				gtk_list_store_set(
-					GTK_LIST_STORE(model), &iter, 
-					AUDIOCD_COL_ICON, icon, 
-					AUDIOCD_COL_FILE, (gchar*)filename, 
-					AUDIOCD_COL_DURATION, info->formattedduration->str,
-					/*AUDIOCD_COL_SIZE, info->filesize,*/
-					AUDIOCD_COL_ARTIST, info->artist->str, 
-					AUDIOCD_COL_ALBUM, info->album->str,
-					AUDIOCD_COL_TITLE, info->title->str, -1);
-				
-				g_object_unref(icon);
-			}
-			else
-			{
-				audioinfo_delete(info);
-				break;
-			}
-			
-			audioinfo_delete(info);
+            GDir* dir = g_dir_open(filename, 0, NULL);
+            const gchar *name = g_dir_read_name(dir);
+            while((name != NULL) && cont)
+            {
+                gchar* fullname = g_build_filename(filename, name, NULL);
+                if(!g_file_test(fullname, G_FILE_TEST_IS_DIR))
+                    cont = audiocd_add_file(fullname, model);
+                g_free(fullname);
+                name = g_dir_read_name(dir);					
+            }
+            g_dir_close(dir);
+        }
+		else
+		{
+			cont = audiocd_add_file(filename, model);
 		}
 		
-		g_free(filename);
-		
+		g_free(filename);		
 		file = strtok(NULL, "\n");
 	}
 	
 	gnomebaker_show_busy_cursor(FALSE);
+		
 }
 
 
@@ -499,3 +625,19 @@ audiocd_import_session()
 {
 	GB_LOG_FUNC
 }
+
+
+void 
+audiocd_move_selected_up()
+{
+    GB_LOG_FUNC
+    audiocd_move_selected(TRUE);
+}
+
+
+void 
+audiocd_move_selected_down()
+{
+    GB_LOG_FUNC
+    audiocd_move_selected(FALSE);
+}
