@@ -15,7 +15,7 @@
  */
 /*
  * File: audiocd.h
- * Created by: luke_biddell@yahoo.com
+ * Copyright: luke_biddell@yahoo.com
  * Created on: Fri May  7 22:37:25 2004
  */
 
@@ -48,6 +48,7 @@ void audiocd_clear();
 void audiocd_remove();
 void audiocd_add_selection(GtkSelectionData* selection);
 void audiocd_import_session();
-							  
+void audiocd_move_selected_up();
+void audiocd_move_selected_down();
 							  
 #endif	/*_AUDIOCD_H_*/
