@@ -15,7 +15,7 @@
  */
  /*
  * File: datacd.h
- * Created by: Luke Biddell <Email>
+ * Copyright: luke_biddell@yahoo.com
  * Created on: Sun May  9 15:16:08 2004
  */
 
@@ -34,13 +34,27 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 gdouble datadisksize = 0.0;
+GtkCellRenderer* contentrenderer = NULL;
 
-DiskSize datadisksizes[] = 
+enum
 {
-	{200 * 1000 * 1000, "200MB CD"},
-	{600 * 1000 * 1000, "600MB CD"},
-	{700 * 1000 * 1000, "700MB CD"},
-	{4.7 * 1000 * 1000 * 1000, "4.7GB DVD"},
+    CD_200MB = 0,
+    CD_650MB,
+    CD_700MB,
+    CD_800MB,
+    DVD_4GB,
+    DVD_8GB,
+    DISK_SIZE_COUNT
+};
+
+
+DiskSize datadisksizes[DISK_SIZE_COUNT] = 
+{
+	{200 * 1024 * 1024, "200MB CD"},
+	{650 * 1024 * 1024, "650MB CD"},
+	{700 * 1024 * 1024, "700MB CD"},
+	{800 * 1024 * 1024, "800MB CD"},
+	{4.7 * 1000 * 1000 * 1000, "4.7GB DVD"}, /* DVDs are salesman's MegaByte ie 1000 not 1024 */
 	{8.5 * 1000 * 1000 * 1000, "8.5GB DVD"}
 };
 
@@ -159,7 +173,7 @@ datacd_update_progress_bar(gboolean add, guint64 filesize)
 	}
 	else
 	{
-		gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
+		gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
 			_("File or directory is too large to fit in the remaining space on the CD"));
 		ok = FALSE;
 	}
@@ -176,7 +190,7 @@ datacd_add_to_compilation(const gchar* file, GtkListStore* liststore, gboolean e
 	g_return_val_if_fail(liststore != NULL, FALSE);
 	gboolean ret = TRUE;
 	
-	gchar* filename = gbcommon_tidy_nautilus_dnd_file(file);
+	gchar* filename = gbcommon_get_local_path(file);
 		
 	GB_DECLARE_STRUCT(struct stat, s);
 	gint statret = stat(filename, &s);
@@ -199,7 +213,7 @@ datacd_add_to_compilation(const gchar* file, GtkListStore* liststore, gboolean e
 			} 
 			else 
 			{
-				gchar* mime = gnome_vfs_get_mime_type(filename);
+				gchar* mime = gbcommon_get_mime_type(filename);
 				icon = gbcommon_get_icon_for_mime(mime, 16);
 				g_free(mime);
 			}
@@ -217,7 +231,7 @@ datacd_add_to_compilation(const gchar* file, GtkListStore* liststore, gboolean e
 	}
 	else
 	{
-		g_warning(_("failed to stat file [%s] with ret code [%d] error no [%s]"), 
+		g_warning("failed to stat file [%s] with ret code [%d] error no [%s]", 
 			filename, statret, strerror(errno));
 		
 		ret = FALSE;
@@ -242,7 +256,7 @@ datacd_add_selection(GtkSelectionData* selection)
 	
 	gnomebaker_show_busy_cursor(TRUE);	    	
 
-	g_message( _("received sel %s"), selection->data);	
+	GB_TRACE("received sel %s", selection->data);	
 	const gchar* file = strtok((gchar*)selection->data,"\n");
 	while(file != NULL)
 	{
@@ -266,7 +280,7 @@ datacd_on_drag_data_received(
     guint time,
     gpointer userdata)
 {
-	GB_LOG_FUNC	
+	GB_LOG_FUNC		
  	datacd_add_selection(seldata);
 }
 
@@ -304,7 +318,7 @@ datacd_remove()
 	gtk_tree_selection_selected_foreach(selection, datacd_foreach_fileselection, &rr_list);		
 	
 	GList *node = rr_list;
-	while(node != NULL)
+	for (; node != NULL ; node = node->next)
 	{
 		if(node->data)
 		{
@@ -332,7 +346,6 @@ datacd_remove()
 				/* FIXME/CHECK: Do we need to free the path here? */
 			}
 		}
-		node = node->next;
 	}
 	
 	g_list_foreach(rr_list, (GFunc)gtk_tree_row_reference_free, NULL);
@@ -408,6 +421,15 @@ datacd_contents_cell_edited(GtkCellRendererText *cell,
 }
 
 
+void 
+datacd_on_edit(gpointer widget, gpointer user_data)
+{
+	GB_LOG_FUNC
+	g_return_if_fail(user_data != NULL);        
+    //g_signal_emit_by_name (contentrenderer, "edited", NULL, NULL);    
+}
+
+
 gboolean
 datacd_on_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
@@ -416,18 +438,20 @@ datacd_on_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user
 	/* look for a right click */	
 	if(event->button == 3)
 	{
-		GtkWidget* menu = gtk_menu_new();	
-		
-		GtkWidget* menuitem = gtk_menu_item_new_with_label(_("Remove selected"));	
-		g_signal_connect(menuitem, "activate",
+		GtkWidget* menu = gtk_menu_new();			
+        GtkTreeView* view = (GtkTreeView*)widget;
+		GtkTreeSelection* selection = gtk_tree_view_get_selection(view);
+		const gint count = gtk_tree_selection_count_selected_rows(selection);
+		if(count == 1)
+		{
+			gbcommon_append_menu_item_stock(menu, _("_Edit name"), GTK_STOCK_DND, 
+				(GCallback)datacd_on_edit, view);
+            gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
+		}
+		gbcommon_append_menu_item_stock(menu, _("_Remove selected"), GTK_STOCK_REMOVE, 
 			(GCallback)datacd_on_remove_clicked, widget);	
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);	
-		
-		menuitem = gtk_menu_item_new_with_label(_("Clear"));	
-		g_signal_connect(menuitem, "activate",
-			(GCallback)datacd_on_clear_clicked, widget);	
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);	
-		
+		gbcommon_append_menu_item_stock(menu, _("Clear"), GTK_STOCK_CLEAR, 
+			(GCallback)datacd_on_clear_clicked, widget);		
 		gtk_widget_show_all(menu);
 	
 		/* Note: event can be NULL here when called. However,
@@ -467,12 +491,12 @@ datacd_new()
 	g_value_init(&value, G_TYPE_BOOLEAN);
 	g_value_set_boolean(&value, TRUE);	
 
-    renderer = gtk_cell_renderer_text_new();
-	g_object_set_property(G_OBJECT(renderer), "editable", &value);
-	g_signal_connect(renderer, "edited", (GCallback)datacd_contents_cell_edited, 
+    contentrenderer = gtk_cell_renderer_text_new();
+	g_object_set_property(G_OBJECT(contentrenderer), "editable", &value);
+	g_signal_connect(contentrenderer, "edited", (GCallback)datacd_contents_cell_edited, 
 		(gpointer)store);
-    gtk_tree_view_column_pack_start(col, renderer, TRUE);
-    gtk_tree_view_column_set_attributes(col, renderer, "text", DATACD_COL_FILE, NULL);
+    gtk_tree_view_column_pack_start(col, contentrenderer, TRUE);
+    gtk_tree_view_column_set_attributes(col, contentrenderer, "text", DATACD_COL_FILE, NULL);
 	gtk_tree_view_append_column(filelist, col);	
 	
 	g_value_unset(&value);
@@ -490,7 +514,7 @@ datacd_new()
 
 	/* Third column to display the human size of file/dir */
 	col = gtk_tree_view_column_new();
-	gtk_tree_view_column_set_title(col, "Size");
+	gtk_tree_view_column_set_title(col, _("Size"));
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(col, renderer, TRUE);
 	gtk_tree_view_column_set_attributes(col, renderer, "text", DATACD_COL_HUMANSIZE, NULL);
@@ -535,7 +559,7 @@ datacd_new()
 	
 	GtkWidget* optmenu = glade_xml_get_widget(gnomebaker_getxml(), widget_datacd_size);
 	gbcommon_populate_disk_size_option_menu(GTK_OPTION_MENU(optmenu), datadisksizes, 
-		(sizeof(datadisksizes)/sizeof(DiskSize)), 2);
+		DISK_SIZE_COUNT, 2);
 	
 	/* get the currently selected cd size */
 	datacd_get_datadisk_size();	
@@ -556,13 +580,13 @@ datacd_get_msinfo(gchar** msinfo)
 	{
 		gchar* message = g_strdup_printf(_("Error getting session information.\n\n%s"), 
 			output != NULL ? output->str : _("unknown error"));
-		gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
+		gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
 		g_free(message);
 	}
 	else
 	{
 		*msinfo = g_strdup_printf("%d,%d", start, end);
-		g_message(_("datacd next session is [%s]"), *msinfo);		
+		GB_TRACE("datacd next session is [%s]", *msinfo);		
 		ok = TRUE;
 	}
 	
@@ -593,11 +617,11 @@ datacd_import_session()
 	gchar* msinfo = NULL;
 	if(!datacd_get_msinfo(&msinfo))
 	{
-		g_critical(_("Error getting msinfo"));
+		g_critical("Error getting msinfo");
 	}
 	else if(!devices_mount_device(GB_WRITER, &mountpoint))
 	{
-		g_critical(_("Error mounting writer device"));
+		g_critical("Error mounting writer device");
 	}
 	else	
 	{							
@@ -614,8 +638,10 @@ datacd_import_session()
 			while(name != NULL)
 			{
 				gchar* fullname = g_build_filename(mountpoint, name, NULL);
-				if(!datacd_add_to_compilation(fullname, model, TRUE))
+				gchar* uri = gbcommon_get_uri(fullname);
+				if(!datacd_add_to_compilation(uri, model, TRUE))
 					break;
+				g_free(uri);
 				g_free(fullname);
 				name = g_dir_read_name(dir);				
 			}
@@ -626,12 +652,12 @@ datacd_import_session()
 		{
 			gchar* message = g_strdup_printf(_("Error importing session from [%s]. "
 				"Please check the device configuration in preferences."), mountpoint);
-			gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
+			gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
 			g_free(message);			
 		}				
 		
 		if(!devices_mount_device(GB_WRITER, NULL))
-			g_critical(_("Error unmounting writer device"));	
+			g_critical("Error unmounting writer device");	
 	}
 	
 	g_free(mountpoint);	
@@ -675,7 +701,7 @@ datacd_on_create_datadisk(gpointer widget, gpointer user_data)
 	GtkTreeModel* datamodel = gtk_tree_view_get_model(GTK_TREE_VIEW(datatree));
 	g_return_if_fail(datamodel != NULL);
 
-	if(datadisksize >= 4.7 * 1000 * 1000 * 1000)
+	if(datadisksize >= datadisksizes[DVD_4GB].size)
 		burn_create_data_dvd(datamodel);
 	else
 		burn_create_data_cd(datamodel);	
