@@ -15,7 +15,7 @@
  */
 /*
  * File: devices.h
- * Created by: luke <luke@dhcp-45-369>
+ * Copyright: luke_biddell@yahoo.com
  * Created on: Mon Feb 24 21:51:18 2003
  */
 
@@ -27,6 +27,23 @@
 #include <glib/gprintf.h>
 #include "preferences.h"
 #include "gnomebaker.h"
+#include <sys/types.h>
+#include <sys/ioctl.h>
+#include <fcntl.h>
+#include <stdlib.h>
+
+#ifdef __FreeBSD__
+#include <sys/cdio.h>
+#define CDROM_DRIVE_STATUS     0x5326
+#define CDS_NO_DISC        1
+#define CDS_TRAY_OPEN      2
+#define CDS_DRIVE_NOT_READY    3
+#define CDS_DISC_OK        4
+#define CDSL_CURRENT       ((int) (~0U>>1))
+#else
+#include <linux/cdrom.h>
+#endif
+
 
 gint deviceadditionindex = 0;
 
@@ -71,8 +88,7 @@ devices_write_device_to_gconf(const gint devicenumber, const gchar* devicename,
 	g_free(devicenodekey);
 	g_free(devicemountkey);
 	g_free(devicecapabilitieskey);
-
-	g_message(_("devices_write_device_to_gconf - Added [%s] [%s] [%s] [%s]"), 
+	GB_TRACE("devices_write_device_to_gconf - Added [%s] [%s] [%s] [%s]", 
 		devicename, deviceid, devicenode, mountpoint);
 }
 
@@ -98,7 +114,7 @@ devices_add_device(const gchar* devicename, const gchar* deviceid,
 			gchar node[64], mount[64];
 			if(sscanf(*line, "%s\t%s", node, mount) == 2)
 			{
-				g_message(_("node [%s] mount [%s]"), node, mount);
+				GB_TRACE("node [%s] mount [%s]", node, mount);
 				if(g_ascii_strcasecmp(node, devicenode) == 0)
 				{
 					mountpoint = g_strdup(mount);
@@ -106,11 +122,12 @@ devices_add_device(const gchar* devicename, const gchar* deviceid,
 				else
 				{
 					/* try to resolve the devicenode in case it's a
-						symlink to the device we are looking for */
-					gchar* linktarget = g_file_read_link(node, NULL);
-					if((linktarget != NULL) && (g_ascii_strcasecmp(linktarget, devicenode) == 0))
+						symlink to the device we are looking for */					
+					gchar* linktarget = g_new0(gchar, PATH_MAX);
+					realpath(node, linktarget);					
+					if(g_ascii_strcasecmp(linktarget, devicenode) == 0)
 					{					
-						g_message(_("node [%s] is link to [%s]"), node, linktarget);
+						GB_TRACE("node [%s] is link to [%s]", node, linktarget);
 						mountpoint = g_strdup(mount);
 					}
 					g_free(linktarget);
@@ -171,7 +188,7 @@ devices_populate_optionmenu(GtkWidget* option_menu, const gchar* devicekey)
 	gint index = 0, history = 0;
 	GSList* devices = preferences_get_key_subkeys(GB_DEVICES_KEY);
 	GSList* item = devices;	
-	while(item)
+	for(; item != NULL; item = item->next)
 	{
 		gchar* devicekey = (gchar*)item->data;		
 		gchar* devicenamekey = g_strconcat(devicekey, GB_DEVICE_NAME_LABEL, NULL);
@@ -193,9 +210,7 @@ devices_populate_optionmenu(GtkWidget* option_menu, const gchar* devicekey)
 		
 		g_free(devicekey);
 		g_free(devicenamekey);		
-		
 		++index;
-		item = item->next;
 	}
 	
 	g_slist_free(devices);
@@ -230,7 +245,7 @@ devices_clear_devicedata()
 
 	GSList* devices = preferences_get_key_subkeys(GB_DEVICES_KEY);
 	GSList* item = devices;	
-	while(item)
+	for(;item != NULL; item = item->next)
 	{
 		gchar* devicekey = (gchar*)item->data;				
 		gchar* devicenamekey = g_strconcat(devicekey, GB_DEVICE_NAME_LABEL, NULL);
@@ -251,8 +266,7 @@ devices_clear_devicedata()
 		g_free(devicenodekey);
 		g_free(devicemountkey);
 		g_free(devicecapabilitieskey);
-		g_free(devicekey);
-		item = item->next;
+		g_free(devicekey);		
 	}
 	
 	g_slist_free(devices);		
@@ -310,7 +324,37 @@ devices_parse_cdrecord_output(const gchar* buffer, const gchar* busname)
 	
 	return ok;
 }
-
+/*
+gboolean
+devices_parse_cdrecord_max_speed(const gchar* buffer, const gchar* busname)
+{
+	GB_LOG_FUNC
+	g_return_val_if_fail(buffer != NULL, FALSE);
+	g_return_val_if_fail(busname != NULL, FALSE);
+	gboolean ok = TRUE;		
+		
+	gchar** lines = g_strsplit(buffer, "\n", 0);
+	gchar** line = lines;
+	while(*line != NULL)
+	{
+		
+		const gchar* maxspeed = strstr(buf, "Maximum write speed:");
+		if(maxspeed != NULL)
+		{
+			gint maxwritespeed = 0;
+			if(sscanf(*line, "%*s %d)", &maxwritespeed) == 1) 
+			{
+			
+			}
+		}
+		++line;
+	}
+	
+	g_strfreev (lines);
+	
+	return ok;
+}
+*/
 
 gboolean 
 devices_probe_bus(const gchar* bus)
@@ -328,9 +372,9 @@ devices_probe_bus(const gchar* bus)
 	
 	GString* buffer = exec_run_cmd(command);
 	if(buffer == NULL)
-		g_critical(_("devices_probe_bus - Failed to scan the scsi bus"));
+		g_critical("devices_probe_bus - Failed to scan the scsi bus");
 	else if(!devices_parse_cdrecord_output(buffer->str, bus))	
-		g_critical(_("devices_probe_bus - failed to parse cdrecord output"));
+		g_critical("devices_probe_bus - failed to parse cdrecord output");
 	else
 		ok = TRUE;
 	
@@ -348,8 +392,7 @@ devices_get_ide_device(const gchar* devicenode, const gchar* devicenodepath,
 	g_return_if_fail(devicenode != NULL);	
 	g_return_if_fail(modelname != NULL);
 	g_return_if_fail(deviceid != NULL);
-	g_message(_("devices_add_ide_device - probing [%s]"), devicenode);
-
+	GB_TRACE("devices_get_ide_device - probing [%s]", devicenode);
 	gchar* contents = NULL;
 	gchar* file = g_strdup_printf("/proc/ide/%s/model", devicenode);
 	if(g_file_get_contents(file, &contents, NULL, NULL))
@@ -361,7 +404,7 @@ devices_get_ide_device(const gchar* devicenode, const gchar* devicenodepath,
 	}
 	else
 	{
-		g_critical(_("Failed to open %s"), file);
+		g_critical("Failed to open %s", file);
 	}
 	g_free(file);
 }
@@ -375,16 +418,16 @@ devices_get_scsi_device(const gchar* devicenode, const gchar* devicenodepath,
 	g_return_if_fail(devicenode != NULL);
 	g_return_if_fail(modelname != NULL);
 	g_return_if_fail(deviceid != NULL);
-	g_message(_("devices_add_scsi_device - probing [%s]"), devicenode);
+	GB_TRACE("devices_add_scsi_device - probing [%s]", devicenode);
 	
 	gchar **device_strs = NULL, **devices = NULL;	
 	if((devices = gbcommon_get_file_as_list("/proc/scsi/sg/devices")) == NULL)
 	{
-		g_critical(_("Failed to open /proc/scsi/sg/devices"));
+		g_critical("Failed to open /proc/scsi/sg/devices");
 	}
 	else if((device_strs = gbcommon_get_file_as_list("/proc/scsi/sg/device_strs")) == NULL)
 	{
-		g_critical(_("Failed to open /proc/scsi/sg/device_strs"));
+		g_critical("Failed to open /proc/scsi/sg/device_strs");
 	}
 	else
 	{
@@ -400,7 +443,7 @@ devices_get_scsi_device(const gchar* devicenode, const gchar* devicenodepath,
 				if(sscanf(*device, "%d\t%*d\t%d\t%d\t%d", 
 					&scsihost, &scsiid, &scsilun, &scsitype) != 4)
 				{
-					g_critical(_("Error reading scsi information from /proc/scsi/sg/devices"));
+					g_critical("Error reading scsi information from /proc/scsi/sg/devices");
 				}			
 				/* 5 is the magic number according to lib-nautilus-burn */
 				else if(scsitype == 5)
@@ -438,7 +481,7 @@ devices_get_scsi_device(const gchar* devicenode, const gchar* devicenodepath,
 void 
 devices_for_each(gpointer key, gpointer value, gpointer user_data)
 {	
-	g_message(_("---- key [%s], value [%s]"), (gchar*)key, (gchar*)value);
+	GB_TRACE("---- key [%s], value [%s]", (gchar*)key, (gchar*)value);
 	g_free(key);
 	g_free(value);
 }
@@ -451,8 +494,7 @@ devices_get_cdrominfo(gchar** proccdrominfo, gint deviceindex)
 	g_return_val_if_fail(proccdrominfo != NULL, NULL);
 	g_return_val_if_fail(deviceindex >= 1, NULL);
 	
-	g_message(_("looking for device [%d]"), deviceindex);
-	
+	GB_TRACE("looking for device [%d]", deviceindex);
 	GHashTable* ret = NULL;
 	gchar** info = proccdrominfo;
 	while(*info != NULL)
@@ -487,8 +529,8 @@ devices_get_cdrominfo(gchar** proccdrominfo, gint deviceindex)
 				 looking for */
 				if(columnindex <= deviceindex)
 				{
-					g_message(_("Requested device index [%d] is out of bounds. "
-						"All devices have been read."), deviceindex);
+					GB_TRACE("Requested device index [%d] is out of bounds. "
+						"All devices have been read.", deviceindex);
 					g_hash_table_destroy(ret);
 					ret = NULL;
 					break;
@@ -518,7 +560,7 @@ devices_probe_busses()
 	gchar **info = NULL;
 	if((info = gbcommon_get_file_as_list("/proc/sys/dev/cdrom/info")) == NULL)
 	{
-		g_critical(_("Failed to open /proc/sys/dev/cdrom/info"));
+		g_critical("Failed to open /proc/sys/dev/cdrom/info");
 	}
 	else
 	{
@@ -583,10 +625,13 @@ devices_mount_device(const gchar* devicekey, gchar** mountpoint)
 	gchar* mount = devices_get_device_config(devicekey, GB_DEVICE_MOUNT_LABEL);		
 	if((mount == NULL) || (strlen(mount) == 0))
 	{
-		gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
-			_("The mount point (e.g. /mnt/cdrom) for the writing device could not be obtained. "
-			"Please check that the writing device has an entry in /etc/fstab and then go "
-			"to preferences and rescan for devices."));
+        if(mountpoint != NULL)
+        {
+            gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE,
+                _("The mount point (e.g. /mnt/cdrom) for the writing device could not be obtained. "
+                "Please check that the writing device has an entry in /etc/fstab and then go "
+                "to preferences and rescan for devices."));
+        }
 	}
 	else
 	{
@@ -603,7 +648,7 @@ devices_mount_device(const gchar* devicekey, gchar** mountpoint)
 		{
 			gchar* message = g_strdup_printf(_("Error %s %s.\n\n%s"), 
 				mount ? _("mounting") : _("unmounting"), mount, output != NULL ? output->str : _("unknown error"));
-			gnomebaker_show_msg_dlg(GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
+			gnomebaker_show_msg_dlg(NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, GTK_BUTTONS_NONE, message);
 			g_free(message);
 		}
 		else
@@ -621,44 +666,103 @@ devices_mount_device(const gchar* devicekey, gchar** mountpoint)
 	return ok;
 }
 
-
-	/* Get the kernel version so that we can figure out what to scan 
-	gfloat version = 0.0;
-	FILE* file = fopen("/proc/sys/kernel/osrelease", "r");
-	if(file == NULL || fscanf(file, "%f.%*d-%*d-%*d", &version) != 1) 
+	
+gboolean
+devices_eject_cd(const gchar* devicekey)
+{
+	/* from http://leapster.org/linux/cdrom/ */
+	gboolean ret = FALSE;
+	gchar *device = devices_get_device_config(devicekey,GB_DEVICE_NODE_LABEL);
+	int cdrom;
+	GB_TRACE("Ejecting media in %s",device);
+	
+	if ((cdrom = open(device,O_RDONLY | O_NONBLOCK)) < 0)
 	{
-		version = 0.0;
-		g_critical("Error getting the kernel version, we will now scan everything.");
+        g_warning("Error opening device %s",device);
+		return FALSE;
+   	}
+			
+
+    /* Use ioctl to send the CDROMEJECT (CDIOCEJECT on FreeBSD) command to the device */
+#ifdef __FreeBSD__
+	if (ioctl(cdrom,CDIOCEJECT,0)<0)
+#else 
+    if (ioctl(cdrom,CDROMEJECT,0)<0)    
+#endif    
+	{
+#ifdef __FreeBSD__
+        return TRUE;
+#else
+        g_critical("devices_query_cdstatus - ioctl failed");
+        return FALSE;
+#endif
+
+    }
+	else
+		ret = TRUE;
+	close(cdrom);
+	return ret;
+}
+/*
+gboolean 
+devices_get_max_speed_for_drive(const gchar* drive)
+{
+	GB_LOG_FUNC
+	gboolean ok = FALSE;
+	g_return_val_if_fail(bus != NULL, FALSE);
+		
+	gchar command[32] = "cdrecord -prcap";
+	if(g_ascii_strncasecmp(bus, "SCSI", 4) != 0)
+	{
+		strcat(command, " dev=");
+		strcat(command, bus);
 	}
-	fclose(file);
 	
-	g_message("kernel version is [%f]", version);
-		
-	if(version > 2.4 || version == 0.0)
-	{	
-		const gchar deviceletters[] = "abcdefghijklmnopqrstuvwxyz";
-		gint i = 0;
-		for(; i < strlen(deviceletters); i++)
-		{
-			gchar* devlabel = g_strdup_printf("/dev/hd%c", deviceletters[i]);
-			ok = devices_probe_bus(devlabel);
-			g_free(devlabel);
-		}
-		
-		i = 0;
-		for(; i <= 16; i++)
-		{
-			gchar* devlabel = g_strdup_printf("/dev/sg%d", i);
-			ok = devices_probe_bus(devlabel);
-			g_free(devlabel);
-		}
-	}*/
+	GString* buffer = exec_run_cmd(command);
+	if(buffer == NULL)
+		g_critical("devices_get_max_speed_for_drive - Failed to scan the scsi bus");
+	else if(!devices_parse_cdrecord_output(buffer->str, bus))	
+		g_critical("devices_get_max_speed_for_drive - failed to parse cdrecord output");
+	else
+		ok = TRUE;
 	
-/*	
-	devices_new_devicedata("'SAMSUNG ' 'CD-ROM SC-152L  ' 'C100' Removable CD-ROM", "1,0,0", "ATA");	
-	devices_new_devicedata("'YAMAHA  ' 'CRW6416S        ' '1.0c' Removable CD-ROM", "0,0,0", NULL);
-	devices_new_devicedata("'_NEC    ' 'DVD_RW ND-3500AG' '2.16' Removable CD-ROM", "1,1,0", "/dev/hdd");
-	devices_new_devicedata("'SONY    ' 'CD-RW  CRX215E1 ' 'SYS2' Removable CD-ROM", "0,1,0", NULL);
-	devices_new_devicedata("'ATAPI   ' 'DVD DUAL 8X4X12 ' 'B3IC' Removable CD-ROM", "1,0,0", NULL);
-	'AOPEN   ' 'CD-RW CRW1232PRO' '1.00' Removable CD-ROM
+	g_string_free(buffer, TRUE);
+
+	return ok;
+}
 */
+
+gboolean
+devices_query_cdstatus(const gchar* devicekey)
+{
+	g_return_val_if_fail(devicekey != NULL,FALSE);
+
+	gboolean retval = FALSE;
+	gchar *device = devices_get_device_config(devicekey,GB_DEVICE_NODE_LABEL);
+		
+	int fd = open(device, O_RDONLY | O_NONBLOCK);
+    
+    int ret = ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
+    if (ret == -1)
+	{
+		g_critical("devices_query_cdstatus - ioctl failed");
+           return FALSE;
+    }
+	switch (ret)
+	{
+    	case CDS_NO_DISC:
+			retval = FALSE;
+				break;
+		case CDS_TRAY_OPEN:
+			retval = FALSE;
+				break;
+		case CDS_DRIVE_NOT_READY:
+			retval = FALSE;
+				break;
+		case CDS_DISC_OK:
+			retval = TRUE;
+				break;
+	}
+	return retval;
+	
+}
