@@ -15,7 +15,7 @@
  */
  /*
  * File: progressdlg.h
- * Created by: luke_biddell@yahoo.com
+ * Copyright: luke_biddell@yahoo.com
  * Created on: Tue Apr  6 22:51:22 2004
  */
 
@@ -41,5 +41,6 @@ void progressdlg_pulse_stop();
 void progressdlg_increment_exec_number();
 void progressdlg_reset_fraction(gfloat fraction);
 void progressdlg_dismiss();
+GtkWindow* progressdlg_get_window();
 
 #endif	/*_PROGRESSDLG_H_*/
